<?php

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use FiloBlu\Refilo\Remote\Entity\Product;
use Magento\CatalogInventory\Model\Stock;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Select;
use Magento\Framework\Module\Manager;
use Magento\Framework\ObjectManagerInterface;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterface;
use Magento\InventorySalesApi\Api\GetStockBySalesChannelInterface;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Api\Data\WebsiteInterface;
use Magento\Store\Model\StoreManagerInterface;
use Zend_Db;

/**
 * Class StockProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
class StockProvider implements EntityProviderInterface
{

    /** @var null */
    protected $ids;
    /** @var int */
    protected $website;
    /** @var array */
    protected $items = [];
    /** @var Manager */
    protected $moduleManager;
    /** @var ResourceConnection */
    private $resourceConnection;

    /**
     * @var ObjectManagerInterface
     */
    private $objectManager;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * StockProvider constructor.
     * @param Manager $moduleManager
     * @param ObjectManagerInterface $objectManager
     * @param ResourceConnection $resourceConnection
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        Manager $moduleManager,
        ObjectManagerInterface $objectManager,
        ResourceConnection $resourceConnection,
        StoreManagerInterface $storeManager
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->moduleManager = $moduleManager;
        $this->objectManager = $objectManager;
        $this->storeManager = $storeManager;
    }

    /**
     * @return mixed
     */
    public function current()
    {
        $current = current($this->items);

        $product = new Product();
        $product->setId((int)$current['id']);
        $product->setData(Product::INVENTORY, [
            'in_stock'   => (bool)$current['is_salable'],
            'backorders' => (bool)$current['backorders'],
            'saleable'    => (bool)$current['is_salable']
        ]);

        return $product;
    }

    /**
     * @return mixed|void
     */
    public function next()
    {
        return next($this->items);
    }

    public function key()
    {
        return current($this->items)['id'];
    }

    public function valid()
    {
        return current($this->items);
    }

    /**
     *
     */
    public function rewind()
    {
        $stockId = Stock::DEFAULT_STOCK_ID;

        if ($this->moduleManager->isEnabled('Magento_Inventory') && $this->website) {
            /** @var GetStockBySalesChannelInterface $getStockBySalesChannel */
            $getStockBySalesChannel = $this->objectManager->get(GetStockBySalesChannelInterface::class);
            /** @var  SalesChannelInterface $salesChannel */
            $salesChannel = $this->objectManager->create(SalesChannelInterface::class);
            $salesChannel->setType(SalesChannelInterface::TYPE_WEBSITE);
            $salesChannel->setCode($this->storeManager->getWebsite($this->website)->getCode());
            $stock = $getStockBySalesChannel->execute($salesChannel);
            $stockId = $stock->getStockId();
        }

        $select = $this->getSelectForStock($stockId);

        if (!empty($this->ids)) {
            $select->where('csi.product_id IN (' . implode(',', $this->ids) . ')');
        }

        $this->items = $this->resourceConnection->getConnection()->fetchAll($select, [], Zend_Db::FETCH_ASSOC);
    }

    /**
     * @param $stockId
     * @return Select
     */
    protected function getSelectForStock($stockId)
    {
        return $this->resourceConnection->getConnection()
            ->select()
            ->from(
                [
                    's' => "inventory_stock_{$stockId}"
                ],
                [
                    'id'         => 'cpe.entity_id',
                    'quantity'   => 'quantity',
                    'is_salable' => 'is_salable',
                    'backorders' => 'csi.backorders'
                ]
            )->join(
                [
                    'cpe' => 'catalog_product_entity'
                ],
                's.sku = cpe.sku',
                []
            )->join(
                ['csi' => 'cataloginventory_stock_item'],
                ' cpe.entity_id = csi.product_id',
                ['backorders' => 'backorders']
            );
    }

    /**
     * @param array $ids
     * @return $this|EntityProviderInterface
     */
    public function withIds(array $ids): EntityProviderInterface
    {
        $this->ids = $ids;
        return $this;
    }

    /**
     * @param StoreInterface $store
     * @return $this|EntityProviderInterface
     */
    public function forStore(StoreInterface $store): EntityProviderInterface
    {
        $this->website = $store->getWebsiteId();
        return $this;
    }

    /**
     * @param WebsiteInterface $website
     * @return $this|EntityProviderInterface
     */
    public function forWebsite(WebsiteInterface $website): EntityProviderInterface
    {
        $this->website = $website->getId();
        return $this;
    }
}
